/*
   Updated to be uBITX external dsp filter.

   Triumviate Skonk Worx
     W0EB  N5IB  W2CTX
       WWW.W0EB.COM
      EXT DSP FILTER
*/





/*     B A S E D   O N   B E L O W
   The GI1MIC $20 Dollar DSP Audio Filter

   Visit https://gi1mic.github.io/ for information on how to use this code.

   Email via GI1MIC at outlook.com

   Modify "settings.h" to suit your needs


   This project is intended to be used with an amplified speaker to provide DSP audio filtering
   mainly for Radio Amateurs or shortwave radio listeners.

   The project uses a rotary encoder plus two push buttons (one built into the encoder) for
   control plus a SPI display to show a GUI.


   Version 1.0
    Initial release based on my earlier FT-817 Version 2.2 code
    but stripped down to a simplified design making it easier to modify

*/


#include <Encoder.h>
#include "settings.h"
#include "dynamicFilters.h"
#include "dspfilter.h"
#include "display.h"

#include <SPI.h>
#include <EEPROM.h>
#include <FreqMeasureMulti.h>
#include <Adafruit_GFX.h>
#include <TFT_ILI9163C.h>
#include <XPT2046_Touchscreen.h>
TFT_ILI9163C tft = TFT_ILI9163C(cs, dc);
XPT2046_Touchscreen ts(CS_PIN);

extern void Display_Splash( void );
extern void Calibrate_Screen( void );
/*
  #define PRESS_QUICK  50
  #define PRESS_MEDIUM 100
  #define PRESS_LONG   150
  #define PRESS_EXTRA_LONG   200
*/
#define PRESS_QUICK  50
#define PRESS_MEDIUM 150
#define PRESS_LONG   250
#define PRESS_EXTRA_LONG   350

extern int minx;
extern int miny;
extern int maxx;
extern int maxy;

FreqMeasureMulti freq1;
float sum1 = 0;
int count1 = 0;
elapsedMillis timeout;
double cf_freq;

// Limits

double FreqMin[5] = {100, 100, 100, 100, 60};
double FreqMax[5] = {1900, 3000, 3000, 3000, 3000};

double Freq_Max = FreqMax[4];
int Encoder_Delay = 150;

bool   Touched = false;

short int     filterType;
double        freqLow;
double        freqHigh;
double        Last_freqLow;
double        Last_freqHigh;



double        last_diff = 0;
double        signal_diff = 0;

const char*   freqDesc;

unsigned int selected_mode = 0;   // 0 = Center point, 1 = Hi, 2 = Low, 3  = BW



Encoder knob(ENCODER1a, ENCODER1b);
long rotaryOldPosition  = 0;
int rotaryDirection = 0;

bool Display_Mode = true;

int Current_CW_Filter = CW_NORM;
double CW_Filter_Data_Low[5] =  { 750, 900, 850, 750, 500 };
double CW_Filter_Data_High[5] = { 1250, 1100, 1150, 1250, 1500 };
double SSB_Filter_Data_Low = 300;
double SSB_Filter_Data_High = 2700;
double DIG_Filter_Data_Low =  2125;
double DIG_Filter_Data_High = 2275;

int CAL_Minx;
int CAL_Miny;
int CAL_Maxx;
int CAL_Maxy;

int DSP_Counter;
char DSP_DISP[12];


/////////////////////////////////// TOUCH

TS_Point p;
TS_Point pwait;
#define DEBUG
#ifdef DEBUG
char buffr[100];
#endif

int rawxx, rawyy;
int xx;
int yy;



//---------------------------------------------------------------


void Wait_For_Touch_Off( void ) {

  pwait = ts.getPoint();
  while ( pwait.z != 4095 ) {
    delay(10);
    pwait = ts.getPoint();
  }

}


void Wait_For_Touch_On( void ) {

  pwait = ts.getPoint();
  while ( pwait.z == 4095 ) {
    delay(10);
    pwait = ts.getPoint();
  }

}


void Get_Touch( bool wait = true ) {

  delay(10);
  p = ts.getPoint();
  if (p.z != 4095) {

    rawxx = p.x;
    rawyy = p.y;
    xx = map(p.x, CAL_Minx, CAL_Maxx, 0, 159);
    yy = map(p.y, CAL_Miny, CAL_Maxy, 0, 127);
    p.x = xx;
    p.y = yy;

#ifdef DEBUG
    sprintf(buffr, "rawxx=%03d rawyy=%03d", rawxx, rawyy);
    Serial.println("");
    Serial.println(buffr);
    sprintf(buffr, "p.z=%03d p.x=%03d p.y=%03d", p.z, p.x, p.y );
    Serial.println(buffr);
#endif
    if ( wait) {
      while (p.z != 4095) {
        p = ts.getPoint();
        delay(10);
      }
    }
    p.x = xx;
    p.y = yy;

  }
}


bool TouchChangedState( void) {
  bool retval = false;

  if ( filterIndex != 4 ) {
    p = ts.getPoint();
    if (p.z != 4095) {
      Get_Touch(false);
      if ( p.x > 15 && p.x < 100 && p.y > 40 && p.y < 70 ) {
        rotaryDirection = -1;
        retval = true;
      } else if ( p.x > 100 && p.x < 160 && p.y > 40 && p.y < 70 ) {
        rotaryDirection = +1;
        retval = true;
      }
    }
  }
  return (retval);
}


void loadFilter() {

  filterType = filterList[filterIndex].filterType;
  if ( filterIndex == 0 ) {
    freqLow = CW_Filter_Data_Low[Current_CW_Filter];
    freqHigh = CW_Filter_Data_High[Current_CW_Filter];
  } else if ( filterIndex == 1 ) {
    freqLow = SSB_Filter_Data_Low;
    freqHigh = SSB_Filter_Data_High;
  } else if ( filterIndex == 2 ) {
    freqLow = DIG_Filter_Data_Low;
    freqHigh = DIG_Filter_Data_High;
    // myFilter2.begin(fir_active2, NUM_COEFFICIENTS);
  } else {
    freqLow = filterList[filterIndex].freqLow;
    freqHigh = filterList[filterIndex].freqHigh;
  }


  freqDesc = filterList[filterIndex].desc;
  if ( filterIndex < 2 ) selected_mode = 0;
  else if ( filterIndex == 2 ) selected_mode = 1;
  else if ( filterIndex == 3 ) selected_mode = 2;

  Freq_Max = FreqMax[filterIndex];
  updateFilter();
  Touched = false;
}

int start_filter = -1;

void setup() {
  char buf[100];
  int temp;

  pinMode(ENCODER1sw, INPUT_PULLUP);

  // get touch calibration parameters
  EEPROM.get(MINX, minx);
  EEPROM.get(MAXX, maxx);
  EEPROM.get(MINY, miny);
  EEPROM.get(MAXY, maxy);
  CAL_Minx = minx;
  CAL_Miny = miny;
  CAL_Maxx = maxx;
  CAL_Maxy = maxy;

  // get saved filter frequencies
  EEPROM.get(CWLO, temp);
  sprintf(buf, "CWLO = %d", temp);
  Serial.println(buf);
  if ( (temp < 100) || (temp > 3000) ) {
    temp = 750;
    // sprintf(buf, "CWLO = %d", temp);
    // Serial.println(buf);
  }
  CW_Filter_Data_Low[0] = temp;

  EEPROM.get(CWHI, temp);
  if ( (temp < 100) || (temp > 3000) ) {
    temp = 1250;
    // sprintf(buf, "CWLO = %d", temp);
    // Serial.println(buf);
  }
  CW_Filter_Data_High[0] = temp;

  EEPROM.get(SSBLO, temp);
  if ( (temp < 100) || (temp > 3000) ) {
    temp = 300;
    // sprintf(buf, "SSBLO = %d", temp);
    // Serial.println(buf);
  }
  SSB_Filter_Data_Low = temp;

  EEPROM.get(SSBHI, temp);
  if ( (temp < 100) || (temp > 3000) ) {
    temp = 2700;
    // sprintf(buf, "SSBHI = %d", temp);
    // Serial.println(buf);
  }
  SSB_Filter_Data_High = temp;

  EEPROM.get(DIGLO, temp);
  if ( (temp < 100) || (temp > 3000) ) {
    temp = 1000;
    // sprintf(buf, "DIGLO = %d", temp);
    // Serial.println(buf);
  }
  DIG_Filter_Data_Low = temp;

  EEPROM.get(DIGHI, temp);
  if ( (temp < 100) || (temp > 3000) ) {
    temp = 1500;
    // sprintf(buf, "DIGHI = %d", temp);
    // Serial.println(buf);
  }
  DIG_Filter_Data_High = temp;

#ifdef DEBUG
  sprintf(buffr, "minx=%03d maxx=%03d", minx, maxx );
  Serial.println("");
  Serial.println(buffr);
  sprintf(buffr, "miny=%03d maxy=%03d", miny, maxy );
  Serial.println(buffr);
#endif




  start_filter = 4;

  DSP_Counter = 0;

  AudioMemory(12);
  tft.begin();
  tft.setRotation(1);
  tft.fillScreen(BLUE);
  ts.begin();
  ts.setRotation(0);
  freq1.begin(6);

  //Press encoder switch to cycle through filters.
  //Rotate encode to move center point of filter.
  //Double press encoder in TPF filter to flip between dual filter.
  //Long press encoder switch to reset TPF");
  delay(500);

  // Down mix stereo to mono on input
  monoMixer.gain(0, 0.5);                 // Left
  monoMixer.gain(1, 0.5);                 // Right
  monoMixer.gain(2, 0.5);                 // PC Left
  monoMixer.gain(3, 0.5);                 // PC Right

  // audioControl.enable();                  // Start the SGTL5000
  // audioControl.inputSelect(AUDIO_INPUT_LINEIN);
  // audioControl.lineInLevel(5);            // Set input level to 1.33V
  // audioControl.lineOutLevel(29);          // Set input level to 1.29V
  // audioControl.volume(0.8);               // Set headset volume level


  // Load default filter
  filterType = filterList[start_filter].filterType;
  freqLow = filterList[start_filter].freqLow;
  freqHigh = filterList[start_filter].freqHigh;
  freqDesc = filterList[start_filter].desc;

  //audioFilter(fir_active1,
  audioFilter(fir_active1,
              NUM_COEFFICIENTS,
              filterType,
              W_HAMMING,
              freqLow,
              freqHigh );
  myFilter1.begin(fir_active1, NUM_COEFFICIENTS);
  loadFilter();

  // Diplsay splash and check for cal request
  Display_Splash();
  if ( !digitalRead(ENCODER1sw) ) {
    tft.setTextSize(1);
    tft.fillScreen(BLACK);
    tft.setTextColor(WHITE);
    tft.setCursor(16, 60);
    tft.print((char *)"Release to start CAL");
    while ( !digitalRead(ENCODER1sw) )delay(10);
    Display_Splash();
    Calibrate_Screen();
  }

  updateDisplay(filterType, freqDesc, freqLow, freqHigh, selected_mode);

}


//---------------------------------------------------------------


void Process_DSP_Filter_Control( void ) {
  bool looper;

  Display_Mode = false;
  delay(100);

  // On touch left button
  p = ts.getPoint();

  if (p.z != 4095) {
    Get_Touch(false);
    if ( p.x > 100 && p.x < 160 && p.y > 10 && p.y < 36 ) {
      Wait_For_Touch_Off();
      if ( filterIndex != 4 ) {
        Display_Mode = true;
        selected_mode++;
        if ( selected_mode > 3 ) selected_mode = 0;
        updateDisplay();
      }
    } else if ( p.x > 15 && p.x < 66 && p.y > 10 && p.y < 36 ) {
      looper = true;
      tft.setTextSize(2);
      tft.setTextColor(WHITE);
      while ( looper ) {
        delay(10);
        p = ts.getPoint();
        if (p.z != 4095) {
          DSP_Counter++;
          if ( DSP_Counter  ==  PRESS_EXTRA_LONG ) {
            tft.fillRect(0, 112, 159, 26, BLUE);
            tft.setCursor(0, 112);
            tft.print("No Change");
          } else if ( DSP_Counter  ==  PRESS_LONG ) {
            tft.fillRect(0, 112, 159, 26, BLUE);
            tft.setCursor(0, 112);
            tft.print("Save Filter");
          } else if ( DSP_Counter  ==   PRESS_MEDIUM ) {
            tft.fillRect(0, 112, 159, 26, BLUE);
            tft.setCursor(0, 112);
            tft.print("Reset Filter");
          } else if ( DSP_Counter  ==   PRESS_QUICK ) {
            tft.fillRect(0, 112, 159, 26, BLUE);
            tft.setCursor(0, 112);
            tft.print("Next Filter");
          }
        } else looper = false;
      }
      // Quick button presses toggles between center, top and low band movement
      if ( DSP_Counter <= PRESS_QUICK ) {
        if ( filterIndex != 4 ) {
          Display_Mode = true;
          selected_mode++;
          if ( selected_mode > 3 ) selected_mode = 0;
          updateDisplay();
        }

      } else if ( DSP_Counter <= PRESS_MEDIUM ) {
        Touched = true;
        loadNextFilter();
        Display_Mode = true;
        updateDisplay();

      } else if ( DSP_Counter < PRESS_LONG ) {
        if ( filterIndex == 0 ) {
          CW_Filter_Data_Low[Current_CW_Filter] = filterList[filterIndex].freqLow;
          CW_Filter_Data_High[Current_CW_Filter] = filterList[filterIndex].freqHigh;
        } else if ( filterIndex == 1 ) {
          SSB_Filter_Data_Low = filterList[filterIndex].freqLow;
          SSB_Filter_Data_High = filterList[filterIndex].freqHigh;
        }

        loadFilter();
        updateDisplay();

      } else if ( DSP_Counter < PRESS_EXTRA_LONG ) {

        if ( filterIndex == 0 ) {
          EEPROM.put(CWLO, int(freqLow));
          EEPROM.put(CWHI, int(freqHigh));
        } else if ( filterIndex == 1 ) {
          EEPROM.put(SSBLO, int(freqLow));
          EEPROM.put(SSBHI, int(freqHigh));
        } else if ( filterIndex == 2 ) {
          EEPROM.put(DIGLO, int(freqLow));
          EEPROM.put(DIGHI, int(freqHigh));
        }

        Display_Mode = true;
        updateDisplay();
      } else updateDisplay();
      DSP_Counter = 0;
    }
  }

  /*
    p = ts.getPoint();

    if (p.z != 4095) {
    Get_Touch(false);
    if ( p.x > 15 && p.x < 66 && p.y > 10 && p.y < 36 ) {
      Wait_For_Touch_Off();
      Touched = true;
      loadNextFilter();
      Display_Mode = true;
      updateDisplay();
    } else if ( p.x > 100 && p.x < 160 && p.y > 10 && p.y < 36 ) {
      Wait_For_Touch_Off();
      if ( filterIndex != 4 ) {
        Display_Mode = true;
        if (filterIndex == 2) selected_mode = 1;
        else if (filterIndex == 3) selected_mode = 2;
        else if ( (filterIndex == 0) && (selected_mode == 0) ) {
          selected_mode = 3;
          updateDisplay();
        }
        else if ( (filterIndex == 0) && (selected_mode == 3) ) {
          selected_mode = 0;
          updateDisplay();
        }
        else {
          selected_mode++;
          if ( selected_mode > 3 ) selected_mode = 0;
          updateDisplay();
        }
      }
    }
    }
  */
  // On button press start measuring time
  if ( !digitalRead(ENCODER1sw) ) {
    tft.setTextSize(2);
    tft.setTextColor(WHITE);
    while ( !digitalRead(ENCODER1sw) ) {
      delay(10);
      DSP_Counter++;
      if ( DSP_Counter  ==  PRESS_EXTRA_LONG ) {
        tft.fillRect(0, 112, 159, 26, BLUE);
        tft.setCursor(0, 112);
        tft.print("No Change");
      } else if ( DSP_Counter  ==  PRESS_LONG ) {
        tft.fillRect(0, 112, 159, 26, BLUE);
        tft.setCursor(0, 112);
        tft.print("Save Filter");
      } else if ( DSP_Counter  ==   PRESS_MEDIUM ) {
        tft.fillRect(0, 112, 159, 26, BLUE);
        tft.setCursor(0, 112);
        tft.print("Reset Filter");
      } else if ( DSP_Counter  ==   PRESS_QUICK ) {
        tft.fillRect(0, 112, 159, 26, BLUE);
        tft.setCursor(0, 112);
        tft.print("Next Filter");
      }
    }
    // Quick button presses toggles between center, top and low band movement
    if ( DSP_Counter <= PRESS_QUICK ) {
      if ( filterIndex != 4 ) {
        Display_Mode = true;
        selected_mode++;
        if ( selected_mode > 3 ) selected_mode = 0;
        updateDisplay();
      }

    } else if ( DSP_Counter <= PRESS_MEDIUM ) {
      loadNextFilter();
      Display_Mode = true;
      updateDisplay();

    } else if ( DSP_Counter < PRESS_LONG ) {
      if ( filterIndex == 0 ) {
        CW_Filter_Data_Low[Current_CW_Filter] = filterList[filterIndex].freqLow;
        CW_Filter_Data_High[Current_CW_Filter] = filterList[filterIndex].freqHigh;
      } else if ( filterIndex == 1 ) {
        SSB_Filter_Data_Low = filterList[filterIndex].freqLow;
        SSB_Filter_Data_High = filterList[filterIndex].freqHigh;
      }

      loadFilter();
      updateDisplay();

    } else if ( DSP_Counter < PRESS_EXTRA_LONG ) {

      if ( filterIndex == 0 ) {
        EEPROM.put(CWLO, int(freqLow));
        EEPROM.put(CWHI, int(freqHigh));
      } else if ( filterIndex == 1 ) {
        EEPROM.put(SSBLO, int(freqLow));
        EEPROM.put(SSBHI, int(freqHigh));
      } else if ( filterIndex == 2 ) {
        EEPROM.put(DIGLO, int(freqLow));
        EEPROM.put(DIGHI, int(freqHigh));
      }

      Display_Mode = true;
      updateDisplay();
    } else updateDisplay();
    DSP_Counter = 0;

  }
}


void Process_DSP_Filter_Computing( void ) {

  if ( filterIndex != 4) {
    if ( (rotaryChangedState() == true) || (TouchChangedState()) ) {
      switch (selected_mode) {
        case 0: // Centerpoint
          updateCenterpoint();
          updateFilter();
          break;
        case 1:  // High
          updateHigh();
          updateFilter();
          break;
        case 2: // Low
          updateLow();
          updateFilter();
          break;
        case 3: // Bandwidth
          updateBW();
          updateFilter();
          break;
        default:
          break;
      };
    };
  }
}



void loop() {
  double calc_diff;
  double current_freq;

  Process_DSP_Filter_Control();
  Process_DSP_Filter_Computing();

  if ( filterIndex == 0 ) {
    if (freq1.available()) {
      sum1 = sum1 + freq1.read();
      count1 = count1 + 1;
    }
    if (timeout > 100) {
      if (count1 > 0) {
        current_freq = freq1.countToFrequency(sum1 / count1);
        Serial.println(current_freq);
        cf_freq = (freqHigh + freqLow) / 2;
        if ( current_freq <= cf_freq ) calc_diff = cf_freq - current_freq;
        else calc_diff = current_freq - cf_freq;
        Serial.print("calc_diff = ");
        Serial.println(calc_diff);

        if ( current_freq == 0 )  tft.fillCircle(78, 15, 13, BLUE);
        else {
          signal_diff = abs(cf_freq -  current_freq);
          Serial.println(signal_diff);
          Serial.println(" ");
          Serial.print(signal_diff);
          Serial.print(" diff ");
          Serial.println(last_diff);
          if (  signal_diff <  last_diff ) tft.fillCircle(78, 15, 13, GREEN);

          if ( signal_diff > 130      )tft.fillCircle(78, 15, 13, BLUE);
          else if ( signal_diff > 120 )tft.fillCircle(78, 15, 12, BLUE);
          else if ( signal_diff > 110 )tft.fillCircle(78, 15, 11, BLUE);
          else if ( signal_diff > 100 )tft.fillCircle(78, 15, 10, BLUE);
          else if ( signal_diff > 90  )tft.fillCircle(78, 15, 9, BLUE);
          else if ( signal_diff > 80  )tft.fillCircle(78, 15, 8, BLUE);
          else if ( signal_diff > 70  )tft.fillCircle(78, 15, 7, BLUE);
          else if ( signal_diff > 60  )tft.fillCircle(78, 15, 6, BLUE);
          else if ( signal_diff > 50  )tft.fillCircle(78, 15, 5, BLUE);
          else if ( signal_diff > 40  )tft.fillCircle(78, 15, 4, BLUE);
          else if ( signal_diff > 30  )tft.fillCircle(78, 15, 3, BLUE);
          else if ( signal_diff > 20  )tft.fillCircle(78, 15, 2, BLUE);
          else if ( signal_diff > 10  )tft.fillCircle(78, 15, 1, BLUE);

          last_diff = signal_diff;
        }
        sum1 = 0;
        count1 = 0;
        timeout = 0;
      } else tft.fillCircle(78, 15, 13, BLUE);
    }
  }
}
//---------------------------------------------------------------
// Load the next filter
void loadNextFilter( ) {
  //  filterIndex++;
  if (filterIndex == 0)filterIndex = 1;
  else if (filterIndex == 1)filterIndex = 2;
  else if (filterIndex == 2)filterIndex = 4;
  else if (filterIndex == 4)filterIndex = 0;
  loadFilter();
}

//---------------------------------------------------------------
// Move the filter center point if encoder is rotated
void updateFilter() {

  audioFilter(fir_active1,
              NUM_COEFFICIENTS,
              filterList[filterIndex].filterType,
              filterList[filterIndex].window,
              freqLow,
              freqHigh);


  updateDisplay();
}


//---------------------------------------------------------------
void updateDisplay() {
  updateDisplay(filterList[filterIndex].filterType,
                filterList[filterIndex].desc,
                freqLow,
                freqHigh,
                selected_mode);
}

//---------------------------------------------------------------
// return if encoder changed state
bool rotaryChangedState() {
  long rotaryNewPosition  = 0;
  bool retval = false;

  //rotaryNewPosition = 0;
  rotaryNewPosition = knob.read();
  knob.write(0);
  // if (rotaryNewPosition !=  0l) {
  if ((rotaryNewPosition > 2l) || (rotaryNewPosition < -2l)) {
    if ( rotaryNewPosition > 2l)rotaryNewPosition = 1;
    else rotaryNewPosition = -1;
    rotaryDirection = rotaryNewPosition;
    retval = true;
    delay(Encoder_Delay);
    Encoder_Delay = 10;
    knob.write(0);
  } else Encoder_Delay = 150;

  return retval;
}

//---------------------------------------------------------------
void updateCenterpoint() {

  Last_freqLow = freqLow;
  Last_freqHigh = freqHigh;

  if (rotaryDirection >= 1) {
    freqHigh  += ENCODER_FREQ_SHIFT;
    if (freqHigh > Freq_Max  ) {
      freqLow = Last_freqLow;
      freqHigh = Last_freqHigh;
    } else freqLow  += ENCODER_FREQ_SHIFT;
  }
  if (rotaryDirection <= -1) {
    freqLow  -= ENCODER_FREQ_SHIFT;
    if (freqLow < FreqMin[filterIndex]  ) {
      freqLow = Last_freqLow;
      freqHigh = Last_freqHigh;
    } else freqHigh  -= ENCODER_FREQ_SHIFT;
  }
}

//---------------------------------------------------------------
void updateHigh() {

  if (rotaryDirection >= 1) {
    freqHigh += ENCODER_FREQ_SHIFT;
    if ( filterIndex == 2 ) {
      if (freqHigh > Freq_Max  ) freqHigh = Freq_Max;
      else if ( (freqHigh - freqLow) > 800 ) freqHigh = freqLow + 800;
    } else {
      if (freqHigh > Freq_Max  ) freqHigh = Freq_Max;
    }
  } else if (rotaryDirection <= -1) {
    freqHigh -= ENCODER_FREQ_SHIFT;
    if ( filterIndex == 2 ) {
      if (freqHigh <= freqLow + 300 )
        freqHigh = freqLow + 300;
    } else {
      if (freqHigh <= freqLow + SMALLEST_BANDWIDTH )
        freqHigh = freqLow + SMALLEST_BANDWIDTH;
    }
  }
}

//---------------------------------------------------------------
void updateLow() {

  if (rotaryDirection >= 1) {
    freqLow += ENCODER_FREQ_SHIFT;
    if ( filterIndex == 2 ) {
      if (freqLow >= freqHigh - FreqMin[filterIndex] )
        freqLow = freqHigh - FreqMin[filterIndex];
      else if ( (freqHigh - freqLow) < 300 ) freqLow = freqHigh - 300;
    } else {
      if (freqLow >= freqHigh - SMALLEST_BANDWIDTH )
        freqLow = freqHigh - SMALLEST_BANDWIDTH;
    }
  } else if (rotaryDirection <= -1) {
    freqLow -= ENCODER_FREQ_SHIFT;
    if ( filterIndex == 2 ) {
      if (freqLow <= FreqMin[filterIndex] )
        freqLow = FreqMin[filterIndex];
      else if ( (freqHigh - freqLow) > 800 ) freqLow = freqHigh - 800;
    } else {
      if (freqLow <= FreqMin[filterIndex] )
        freqLow = FreqMin[filterIndex];
    }
  }
}
//---------------------------------------------------------------
void updateBW() {

  Last_freqLow = freqLow;
  Last_freqHigh = freqHigh;

  if (rotaryDirection >= 1) {
    freqLow -= ENCODER_FREQ_SHIFT / 2;
    freqHigh += ENCODER_FREQ_SHIFT / 2;
    if ( filterIndex == 2 ) {
      if ( (freqHigh - freqLow) > 800 ) {
        freqLow = Last_freqLow;
        freqHigh = Last_freqHigh;
      }
    } else {
      if ( (freqLow < FreqMin[filterIndex]) || (freqHigh > FreqMax[filterIndex])) {
        freqLow = Last_freqLow;
        freqHigh = Freq_Max;
      }
    }
  }
  if (rotaryDirection <= -1) {
    freqLow += ENCODER_FREQ_SHIFT / 2;
    freqHigh -= ENCODER_FREQ_SHIFT / 2;
    if ( filterIndex == 2 ) {
      if ( (freqHigh - freqLow) < 300 ) {
        freqLow = Last_freqLow;
        freqHigh = Last_freqHigh;
      }
    } else {
      if ( (freqHigh - freqLow) < SMALLEST_BANDWIDTH) {
        freqLow = Last_freqLow;
        freqHigh = Last_freqHigh;
      }
    }
  }
}

