#include "settings.h"
#include "display.h"
#include "dynamicFilters.h"



#include <EEPROM.h>
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <TFT_ILI9163C.h>
#include <XPT2046_Touchscreen.h>

#define   DATE "Jan 05, 2019  21:30"
#define   VERSION "Version: 1.00R"


#define DIS_WIDTH  FREQ_MAX
#define DIS_OFFSET (FREQ_MAX - 3000)/2


extern TFT_ILI9163C tft;
extern XPT2046_Touchscreen ts;

extern char DSP_DISP[12];
extern unsigned int filterIndex;
extern double Freq_Max;
extern double Freq_Min;
extern TS_Point p;

extern bool Display_Mode;
extern int CAL_Minx;
extern int CAL_Miny;
extern int CAL_Maxx;
extern int CAL_Maxy;
extern double cf_freq;

extern double CW_Filter_Data_Low;
extern double CW_Filter_Data_High;
extern double SSB_Filter_Data_Low;
extern double SSB_Filter_Data_High;
extern double DIG_Filter_Data_Low;
extern double DIG_Filter_Data_High;

extern double FreqMin[5];
extern double FreqMax[5];



//---------------------------------------------------------------
int minx;
int miny;
int maxx;
int maxy;

bool Zero_Beat = true;
double Zero_Freq = 650;

void Calibrate_Screen( void ) {
  bool looper = true;
  int i;
  int Screen_Calibration_Matrix[16][2];
  char buffr[32];
  //char fred[7];

  tft.setTextSize(1);

  while ( looper ) {
    tft.fillScreen(BLACK);
    // tft.setFont(Arial_14);
    tft.setTextColor(WHITE);
    tft.fillCircle(5, 5, 3, WHITE);
    tft.setCursor(10, 60);
    tft.print((char *)"Dot in upper left corner");
    tft.setCursor(10, 70);
    tft.print((char *)"Press and Hold");
    // get Minx and Miny
    p = ts.getPoint();
    while (p.z == 4095) {
      delay(100);
      p = ts.getPoint();
    }

    i = 0;
    delay(100);
    while ( (i < 10) && (p.z != 4095)) {
      p = ts.getPoint();
      Screen_Calibration_Matrix[i][0] = p.x;
      Screen_Calibration_Matrix[i][1] = p.y;
      i = i + 1;
      delay(100);
      p = ts.getPoint();
    }
    if ( i == 10 ) {
      tft.fillRect(16, 200, 200, 32, BLACK);
      tft.setCursor(16, 200);
      tft.print((char *)"RELEASE NOW");
      p = ts.getPoint();
      while (p.z == 4095) {
        delay(10);
        p = ts.getPoint();
      }
      for (i = 0; i < 10; i++) {
        if (Screen_Calibration_Matrix[i][0] < minx)minx = Screen_Calibration_Matrix[i][0];
        if (Screen_Calibration_Matrix[i][1] < miny)miny = Screen_Calibration_Matrix[i][1];
      }
      if ( minx < 0 )minx = 0;
      if ( miny < 0 )miny = 0;
      looper = false;
    } else {
      tft.setTextColor(RED);
      tft.setCursor(0, 80);
      tft.print((char *)"Failed to hold long enough");
      delay(2000);
    }
  }
  ////////////////////////////////
  looper = true;

  while ( looper ) {
    tft.fillScreen(BLACK);
    //  tft.setFont(Arial_14);
    tft.setTextColor(WHITE);
    tft.fillCircle(156, 117, 3, WHITE);
    tft.setCursor(10, 60);
    tft.print((char *)"Dot lower right corner");
    tft.setCursor(10, 70);
    tft.print((char *)"Press and Hold");
    // get Maxx and Maxy
    p = ts.getPoint();
    while (p.z == 4095) {
      delay(10);
      p = ts.getPoint();
    }

    i = 0;
    delay(100);
    while ( (i < 10) && (p.z != 4095)) {
      p = ts.getPoint();
      Screen_Calibration_Matrix[i][0] = p.x;
      Screen_Calibration_Matrix[i][1] = p.y;
      i = i + 1;
      delay(100);
      p = ts.getPoint();
      while (p.z == 4095) {
        delay(100);
        p = ts.getPoint();
      }
    }
    if ( i == 10 ) {
      tft.fillRect(16, 200, 200, 32, BLACK);
      tft.setCursor(16, 200);
      tft.print((char *)"RELEASE NOW");
      p = ts.getPoint();
      while (p.z == 4095) {
        delay(10);
        p = ts.getPoint();
      }

      for (i = 0; i < 10; i++) {
        if (Screen_Calibration_Matrix[i][0] > maxx)maxx = Screen_Calibration_Matrix[i][0];
        if (Screen_Calibration_Matrix[i][1] > maxy)maxy = Screen_Calibration_Matrix[i][1];
      }
      looper = false;
    } else {
      tft.setTextColor(RED);
      tft.setCursor(0, 80);
      tft.print((char *)"Failed to hold long enough");
      delay(1500);
    }
  }

  /////////////////////////
  tft.fillScreen(BLACK);
  //tft.setFont(Arial_14);
  tft.setTextColor(WHITE);
  sprintf(buffr, "MinX = %d  MaxX = %d", minx, maxx);
  tft.setCursor(0, 10);
  tft.print(buffr);
  sprintf(buffr, "MinY = %d  MaxY = %d", miny, maxy);
  tft.setCursor(0, 20);
  tft.print(buffr);
  tft.setTextColor(YELLOW);
  tft.setCursor(0, 30);
  tft.print("TAP SCREEN TO CONTINUE");
  p = ts.getPoint();
  while (p.z != 4095) {
    delay(100);
    p = ts.getPoint();
  }
  p = ts.getPoint();
  while (p.z == 4095) {
    delay(100);
    p = ts.getPoint();
  }

  EEPROM.put(MINX, minx);
  EEPROM.put(MAXX, maxx);
  EEPROM.put(MINY, miny);
  EEPROM.put(MAXY, maxy);


  CAL_Minx = minx;
  CAL_Miny = miny;
  CAL_Maxx = maxx;
  CAL_Maxy = maxy;



  tft.fillScreen(BLACK);

}

void Display_Splash( void ) {

  tft.setTextColor(BLUE);
  tft.setTextSize(3);
  tft.fillScreen(BLUE);
  tft.fillRect(01, 0, 159, 26, BLUE);
  tft.fillRect(6, 3, 56, 25, WHITE);
  tft.drawRect(4, 1, 58, 27, BLACK);
  tft.drawRect(5, 2, 58, 27, BLACK);
  tft.setCursor(8, 5);
  tft.print("OFF");
  tft.setTextSize(1);
  //    tft.print("   ");
  tft.fillRect(01, 80, 159, 26, BLUE);
  tft.fillRect(0, 32, 160, 56, WHITE);
  tft.setCursor(0, 34);
  tft.setTextColor(RED);
  tft.println("        Triumvirate");
  tft.println("           Skonk");
  tft.println("             Worx");
  tft.setTextColor(BLUE);
  tft.setCursor(0, 66);
  tft.println("      W0EB N5IB W2CTX");
  tft.setTextColor(BLACK);
  tft.setCursor(0, 76);
  tft.println("        WWW.W0EB.COM");
  tft.setCursor(0, 92);
  tft.setTextColor(WHITE);
  tft.println("    EXTERNAL DSP FILTER");
  tft.setCursor(24, 102);
  tft.println(DATE);
  tft.setCursor(38, 114);
  tft.println(VERSION);
  tft.setTextSize(3);

}



void updateDisplay(const int &TYPE, const char *desc, double freqLow, double freqHigh, unsigned int selected_mode) {
  int GRAPH_CONST;

  // Title

  if ( filterIndex == 4 ) {
    Display_Splash();
  } else {
    if ( Display_Mode ) {
      Display_Mode = false;
      tft.fillScreen(BLUE);
      tft.setTextColor(BLUE);
      tft.setTextSize(3);
      //      if ( filterIndex == 2 ) {
      //        tft.fillRect(6, 3, 74, 25, WHITE);
      //        tft.drawRect(4, 1, 76, 27, BLACK);
      //        tft.drawRect(5, 2, 76, 27, BLACK);
      //      } else {
      tft.fillRect(6, 3, 56, 25, WHITE);
      tft.drawRect(4, 1, 58, 27, BLACK);
      tft.drawRect(5, 2, 58, 27, BLACK);
      //      }
      tft.setCursor(8, 5);
      tft.print(desc);
      tft.fillRect(79, 0, 83, 30, BLUE);

      if ( filterIndex < 3 ) {
        //tft.fillRect(6, 3, 56, 25, WHITE);
        tft.fillRect(98, 3, 56, 25, WHITE);
        //tft.drawRect(4, 1, 58, 27, BLACK);
        //tft.drawRect(5, 2, 58, 27, BLACK);

        tft.drawRect(95, 1, 60, 27, BLACK);
        tft.drawRect(96, 2, 60, 27, BLACK);
        tft.setCursor(8, 5);
        tft.print(desc);
        tft.setCursor(100, 5);
        tft.setTextColor(BLUE);
        if ( selected_mode == 0 )tft.print("Ctr");
        else if ( selected_mode == 1 )tft.print("Hi");
        else if ( selected_mode == 2 )tft.print("Lo");
        else if ( selected_mode == 3 )tft.print("BW");
        tft.setTextColor(WHITE);
      }
    }
  }

  if ( filterIndex != 4 ) {

    switch (TYPE) {
      case ID_LOWPASS:
        freqLow = FreqMin[filterIndex];
        break;
      case ID_HIGHPASS:
        freqHigh = FreqMax[filterIndex];
        freqLow = freqLow;
        break;
      case ID_BANDPASS:
        break;
      case ID_BANDSTOP:
        break;
      default:
        break;
    }

    //graph
    GRAPH_CONST = 22;
    if ( filterIndex == 0 ) GRAPH_CONST = 12;
    tft.fillRect(0, 30, 160, 50, BLUE);

    tft.fillRect(0, 72, int(freqLow / GRAPH_CONST), 2, WHITE);

    tft.drawLine(int(freqLow / GRAPH_CONST - 2), 72, int(freqLow / GRAPH_CONST + 2), 32, WHITE);
    tft.drawLine(int(freqLow / GRAPH_CONST - 1), 72, int(freqLow / GRAPH_CONST + 3), 32, WHITE);

    tft.fillRect(int(freqLow / GRAPH_CONST + 4), 32, int(freqHigh / GRAPH_CONST) - int(freqLow / GRAPH_CONST) - 4, 2, WHITE);

    tft.drawLine(int(freqHigh / GRAPH_CONST), 32, int(freqHigh / GRAPH_CONST + 4), 72, WHITE);
    tft.drawLine(int(freqHigh / GRAPH_CONST + 1), 32, int(freqHigh / GRAPH_CONST + 5), 72, WHITE);

    tft.fillRect(int(freqHigh / GRAPH_CONST + 5), 72, 160 - int(freqHigh / GRAPH_CONST + 7), 2, WHITE);

    // Footer
    tft.setTextColor(WHITE);
    tft.setTextSize(2);
    tft.fillRect(0, 78, 160, 20, BLUE);
    itoa(freqHigh, DSP_DISP, 10);
    tft.setCursor(109, 78);
    tft.print(DSP_DISP);

    itoa(freqLow, DSP_DISP, 10);
    tft.setCursor(16, 78);
    tft.print(DSP_DISP);

    tft.fillRect(0, 96, 160, 32, BLUE);
    itoa(freqHigh - freqLow, DSP_DISP, 10);
    tft.setTextSize(1);
    tft.setCursor(2, 100);
    tft.print("BW");
    tft.setTextSize(2);
    tft.setCursor(16, 96);
    tft.print(DSP_DISP);

    cf_freq = (freqHigh + freqLow) / 2;
    itoa(cf_freq, DSP_DISP, 10);
    tft.setTextSize(1);
    tft.setCursor(94, 100);
    tft.print("CF");
    tft.setTextSize(2);
    tft.setCursor(109, 96);
    // if( (filterIndex == 0)&& (Zero_Freq == ((freqHigh + freqLow) / 2)))tft.setTextColor(GREEN);
    tft.print(DSP_DISP);
    tft.setTextColor(WHITE);

    if ( filterIndex == 0 ) {
      CW_Filter_Data_Low = freqLow;
      CW_Filter_Data_High = freqHigh;
    } else if ( filterIndex == 1 ) {
      SSB_Filter_Data_Low = freqLow;
      SSB_Filter_Data_High = freqHigh;
    } else if ( filterIndex == 2 ) {
      DIG_Filter_Data_Low = freqLow;
      DIG_Filter_Data_High = freqHigh;
    }
  }
}
